#define G_LOG_DOMAIN "ap-character"

#include "ap-ability.h"
#include "ap-skill.h"
#include "ap-attack.h"
#include "ap-manager.h"
#include "ap-character.h"

#include <glib.h>

typedef struct _APCharacterPrivate
{
    gchar *name;

    GHashTable *abilities;
    GHashTable *skills;
} APCharacterPrivate;

struct _APCharacter
{
    GObject parent_instance;
};

G_DEFINE_TYPE_WITH_PRIVATE (APCharacter, ap_character, G_TYPE_OBJECT)

enum
{
    PROP_0,
    PROP_NAME,
    N_PROPERTIES
};

static GParamSpec *obj_properties[N_PROPERTIES] = { NULL, };

void
ap_character_add_ability (APCharacter *self,
                          APAbility   *ability)
{
    APCharacterPrivate *priv = ap_character_get_instance_private (self);
    gchar *id;

    id = ap_ability_get_id (ability);

    g_hash_table_insert (priv->abilities, id, ability);
}

GList *
ap_character_get_abilities (APCharacter *self)
{
    APCharacterPrivate *priv = ap_character_get_instance_private (self);

    return g_hash_table_get_values (priv->abilities);
}

void
ap_character_add_skill (APCharacter *self,
                        APSkill     *skill)
{
    APCharacterPrivate *priv = ap_character_get_instance_private (self);
    gchar *name;

    name = ap_skill_get_name (skill);

    g_hash_table_insert (priv->skills, name, skill);
}

GList *
ap_character_get_skills (APCharacter *self)
{
    APCharacterPrivate *priv = ap_character_get_instance_private (self);

    return g_hash_table_get_values (priv->skills);
}

void
ap_character_add_attack (APCharacter *self,
                         APAttack    *attack)
{

}

void
ap_character_print (APCharacter *self)
{
    GList *l;
    GList *abilities;
    GList *skills;

    /* Bio */

    /* Abilities */
    g_print ("Abilities:\n");

    abilities = ap_character_get_abilities (self);

    for (l = abilities; l != NULL; l = l->next)
    {
        g_print ("%s: %d (%d)\n", ap_ability_get_name (l->data), ap_ability_get_score (l->data), ap_ability_get_modifier (l->data));
    }

    g_list_free (abilities);

    g_print ("\n");

    /* Skills */
    g_print ("Skills:\n");

    skills = ap_character_get_skills (self);

    for (l = skills; l != NULL; l = l->next)
    {
        g_print ("%s: %d\n", ap_skill_get_name (l->data), ap_skill_get_score (l->data));
    }

    g_list_free (skills);

    /* Attacks */
}

static void
ap_character_set_property (GObject      *object,
                           guint         property_id,
                           const GValue *value,
                           GParamSpec   *pspec)
{
    APCharacter *self = AP_CHARACTER (object);
    APCharacterPrivate *priv = ap_character_get_instance_private (self);

    switch (property_id)
    {
        case PROP_NAME:
            priv->name = g_value_dup_string (value);
            break;

        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
            break;
    }
}

static void
ap_character_get_property (GObject      *object,
                           guint         property_id,
                           GValue       *value,
                           GParamSpec   *pspec)
{
    APCharacter *self = AP_CHARACTER (object);
    APCharacterPrivate *priv = ap_character_get_instance_private (self);

    switch (property_id)
    {
        case PROP_NAME:
            g_value_set_string (value, priv->name);
            break;

        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
            break;
    }
}

static void
ap_character_finalize (GObject *object)
{
    G_OBJECT_CLASS (ap_character_parent_class)->finalize (object);
}

static void
ap_character_dispose (GObject *object)
{
    G_OBJECT_CLASS (ap_character_parent_class)->finalize (object);
}

static void
ap_character_init (APCharacter *self)
{
    APCharacterPrivate *priv = ap_character_get_instance_private (self);

    priv->abilities = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
    priv->skills = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
}

static void
ap_character_class_init (APCharacterClass *klass)
{
    GObjectClass *object_class = G_OBJECT_CLASS (klass);

    object_class->set_property = ap_character_set_property;
    object_class->get_property = ap_character_get_property;
    object_class->finalize = ap_character_finalize;
    object_class->dispose = ap_character_dispose;

    obj_properties[PROP_NAME] =
        g_param_spec_string ("name",
                             "name",
                             "Name of the character.",
                             NULL  /* default value */,
                             G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE);

    g_object_class_install_properties (object_class,
                                       N_PROPERTIES,
                                       obj_properties);
}

/**
 * ap_character_new:
 *
 * Creates a new #APCharacter, a container for a characters abilties, skills,
 * etc..
 *
 * ## Usage
 * |[
 * APCharacter *character;
 *
 * character = ap_character_new ();
 * ]|
 *
 * Returns: #APCharacter; a sub-object of #GObject
 */
APCharacter *
ap_character_new (void)
{
    return g_object_new (AP_TYPE_CHARACTER,
                         NULL);
}
