#define G_LOG_DOMAIN "ap-manager"

#include "ap-ability.h"
#include "ap-skill.h"
#include "ap-manager.h"

#include <glib.h>
#include <glib-object.h>

typedef struct _APManagerPrivate
{
    GList *abilities;
} APManagerPrivate;

struct _APManager
{
    GObject parent_instance;
};

G_DEFINE_TYPE_WITH_PRIVATE (APManager, ap_manager, G_TYPE_OBJECT)

/* Singleton instance */
APManager *ap_manager_instance = NULL;

enum
{
    PROP_0,
    N_PROPERTIES
};

static GParamSpec *obj_properties[N_PROPERTIES] = { NULL, };

enum
{
    ABILITY_REGISTERED,
    ABILITY_UNREGISTERED,
    SKILL_REGISTERED,
    SKILL_UNREGISTERED,
    ATTACK_REGISTERED,
    ATTACK_UNREGISTERED,
    NUM_SIGNALS
};

static guint signals[NUM_SIGNALS] = { 0, };

static void
ap_manager_finalize (GObject *object)
{
    G_OBJECT_CLASS (ap_manager_parent_class)->finalize (object);
}

static void
ap_manager_init (APManager *self)
{
}

static void
ap_manager_class_init (APManagerClass *klass)
{
    GObjectClass *object_class = G_OBJECT_CLASS (klass);

    object_class->finalize = ap_manager_finalize;

    signals[ABILITY_REGISTERED] =
        g_signal_new ("ability-registered",
                      AP_TYPE_MANAGER,
                      G_SIGNAL_RUN_FIRST,
                      0,
                      NULL,
                      NULL,
                      NULL,
                      G_TYPE_NONE,
                      1,
                      AP_TYPE_ABILITY);

    signals[ABILITY_UNREGISTERED] =
        g_signal_new ("ability-unregistered",
                      AP_TYPE_MANAGER,
                      G_SIGNAL_RUN_FIRST,
                      0,
                      NULL,
                      NULL,
                      NULL,
                      G_TYPE_NONE,
                      1,
                      AP_TYPE_ABILITY);
}

/**
 * ap_manager_new:
 *
 * Creates a new #APManager, manages the connection between #APAbility,
 * #APSkill, #APAttack, etc... #APManager registers and un-registers
 * the various signals.
 *
 * ## Usage
 * |[
 * APManager *manager;
 *
 * manager = ap_manager_new ();
 * ]|
 *
 * Returns: A new #APManager instance
 */
APManager *
ap_manager_new (void)
{
    return g_object_new (AP_TYPE_MANAGER,
                         NULL);
}

/**
 * ap_manager_get_default:
 *
 * Returns the default #APManager instance, if an #APManager instance does not
 * exist then one is created.
 *
 * Returns: Default #APManager instance
 */
APManager *
ap_manager_get_default (void)
{
    if (!ap_manager_instance)
        ap_manager_instance = ap_manager_new ();

    return ap_manager_instance;
}

void
ap_manager_add_ability (APManager *self,
                        APAbility *ability)
{
    APManagerPrivate *priv = ap_manager_get_instance_private (self);

    priv->abilities = g_list_append (priv->abilities, ability);

    g_signal_emit (self, signals[ABILITY_REGISTERED], 0, ability);
}
