#ifndef __AP_SHEET_H__
#define __AP_SHEET_H__

#include <glib.h>
#include <glib-object.h>

G_BEGIN_DECLS

#define AP_TYPE_SHEET (ap_sheet_get_type ())

G_DECLARE_FINAL_TYPE (APSheet, ap_sheet, AP, SHEET, GObject)

typedef struct _APSheetCharacteristic {
    gchar *label;
    gchar *value;
} APSheetCharacteristic;

typedef struct _APSheetAttribute {
    gchar *label;
    gchar *value;
} APSheetAttribute;

typedef struct _APSheetAbility {
    gchar *label;
    gint value;
} APSheetAbility;

typedef struct _APSheetSkill {
    gchar *label;
    gint value;
    gboolean *proficient;
} APSheetSkill;

typedef struct _APSheetSavingThrow {
    gchar *label;
    gint value;
    gboolean *proficient;
} APSheetSavingThrow;

typedef struct _APSheetProficiency {
    gchar *label;
    gchar *value;
} APSheetProficiency;

typedef struct _APSheetescription {
    gchar *label;
    gchar *value;
} APSheetescription;



APSheet *
ap_sheet_new (void);

APSheet *
ap_sheet_from_file (const gchar *filename);

void
ap_sheet_add_characteristic (APSheet *self,
			                 gchar   *key,
                             gchar   *label,
			                 gchar   *value);

APSheetCharacteristic *
ap_sheet_get_characteristic (APSheet *self,
			                 gchar   *key);

gchar *
ap_sheet_get_characteristic_label (APSheet *self,
                                  gchar   *key);

gchar *
ap_sheet_get_characteristic_value (APSheet *self,
                                   gchar   *key);

void
ap_sheet_add_attribute (APSheet *self,
	                    gchar   *key,
                        gchar   *label,
	                    guint    value);

guint *
ap_sheet_get_attribute (APSheet *self,
			            gchar   *key);

void
ap_sheet_add_xp_level (APSheet *self,
                       gchar   *key,
                       gint     value);

void
ap_sheet_add_xp_levels (APSheet  *self,
                        GArray  **levels);

void
ap_sheet_add_ability (APSheet *self,
                      gchar   *key,
                      gchar   *label,
                      gint     value);

void
ap_sheet_add_ability_modifier (APSheet *self,
                               gchar   *key,
                               guint   *value);

void
ap_sheet_add_ability_modifiers (APSheet  *self,
                                GArray  **modifiers);

void
ap_sheet_add_skill (APSheet  *self,
	                gchar    *key,
                    gchar    *label,
	                gint      value,
                    gboolean *proficient);
void
ap_sheet_add_saving_throw (APSheet  *self,
	                       gchar    *key,
                           gchar    *label,
	                       gint      value,
                           gboolean *proficient);

void
ap_sheet_add_proficiency (APSheet  *self,
	                      gchar    *key,
                          gchar    *label,
	                      gchar    *value);

void
ap_sheet_add_proficiencies (APSheet  *self,
	                        gchar    *key,
                            gchar    *label,
                            GArray  **values);

void
ap_sheet_add_description (APSheet  *self,
	                      gchar    *key,
                          gchar    *label,
	                      gchar    *value);


G_END_DECLS

#endif /* __AP_SHEET_H__ */

