#include "test-ability.h"

#include <adventure-pack.h>

#include <glib.h>
#include <locale.h>

int
main (int    argc,
      char **argv)
{
    APManager *manager;
    APCharacter *character;
    APAbility *ability;
    APSkill *skill;
    //APAttack * attack;

    manager = ap_manager_get_default ();

    character = ap_character_new ();

    /* Add Ability */
    ability = g_object_new (AP_TYPE_ABILITY,
                            "id", "strength",
                            "name", "Strength",
                            "score", 15,
                            "bonus", 0,
                            NULL);

    ap_manager_add_ability (manager, ability);
    ap_character_add_ability (character, ability);

    ability = g_object_new (AP_TYPE_ABILITY,
                            "id", "dexterity",
                            "name", "Dexterity",
                            "score", 5,
                            "bonus", 0,
                            NULL);

    ap_character_add_ability (character, ability);

    ability = g_object_new (AP_TYPE_ABILITY,
                            "id", "wisdom",
                            "name", "Wisdom",
                            "score", 10,
                            "bonus", 0,
                            NULL);

    ap_character_add_ability (character, ability);

    /* Add Skill */
    skill = g_object_new (AP_TYPE_SKILL,
                          "id", "acrobatics",
                          "name", "Acrobatics",
                          "modifier", "dexterity",
                          "bonus", 0,
                          "proficient", TRUE,
                           NULL);

    ap_character_add_skill (character, skill);

    skill = g_object_new (AP_TYPE_SKILL,
                          "id", "animal-handling",
                          "name", "Animal Handling",
                          "modifier", "wisdom",
                          "bonus", 0,
                          "proficient", FALSE,
                           NULL);

    ap_character_add_skill (character, skill);

    /* Add Attack */
    /*
    attack = g_object_new (AP_TYPE_ATTACK,
                           "name", "slash",
                           "label", "Slash",
                           "type", "slash",
                           "damage", 10,
                           "critical", 20,
                           "range", 1,
                           "ammo", 0,
                           "bonus", 0,
                           NULL);

    ap_character_add_attack (character, attack);
    */
    ap_character_print (character);

    g_print ("\n");

    setlocale (LC_ALL, "");

    g_test_init (&argc, &argv, NULL);
    g_test_bug_base ("https://github.com/adventure-pack/libadventure-pack/issues");

    g_test_add ("/ability/id", AbilityFixture, NULL,
                ability_fixture_set_up,
                test_ability_id,
                ability_fixture_tear_down);

    g_test_add ("/ability/id_changed", AbilityFixture, NULL,
                ability_fixture_set_up,
                test_ability_id_changed,
                ability_fixture_tear_down);

    g_test_add ("/ability/name", AbilityFixture, NULL,
                ability_fixture_set_up,
                test_ability_name,
                ability_fixture_tear_down);

    g_test_add ("/ability/score", AbilityFixture, NULL,
                ability_fixture_set_up,
                test_ability_score,
                ability_fixture_tear_down);

    g_test_add ("/ability/score-with-bonus", AbilityFixture, NULL,
                ability_fixture_set_up,
                test_ability_score_with_bonus,
                ability_fixture_tear_down);

    for (gint i = 1; i <=  30; i++)
    {
        /*
         * test_name = /ability/modifier/$i
         */
        gchar *test_name = g_strconcat ("/ability/modifier/", g_strdup_printf ("%i", i), NULL);

        g_test_add (test_name, AbilityFixture, GINT_TO_POINTER (i),
                    ability_fixture_set_up,
                    test_ability_modifier,
                    ability_fixture_tear_down);
    }

    return g_test_run ();
}
